/* rndlinux.c  -  raw random number for OSes with /dev/random
 *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
 *
 * This file is part of GnuPG.
 *
 * GnuPG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GnuPG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */


#include <config.h>

#ifdef USE_RNDPOETRY

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "poetry.h"



BOOL CheckRndState(HANDLE hRnd)
{
	char buf[1];
	DWORD dwRead;
	return ReadFile(hRnd, buf, 1, &dwRead, NULL);
}

int rndpoetry_gather_random( void (*add)(const void*, size_t, int), int requester,
					   size_t length, int level )
{
    static HANDLE hRnd = INVALID_HANDLE_VALUE;
	BOOL bMustReseed = FALSE;
	BOOL bRet;
	DWORD n;
    byte buffer[768];


	if(hRnd == INVALID_HANDLE_VALUE)
	{
		hRnd = CreateFileW(L"RND1:", GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if( hRnd == INVALID_HANDLE_VALUE  )
			g10_log_fatal("Can't open the random device\nPlease install Poetry first!");
	}

	if(!CheckRndState(hRnd))
	{
		tty_printf("Poetry has no or too little entropy to generate\ncryptographic strong random numbers!\n");
		tty_printf("\nPlease start Epyc and perform a reseed!\n");

		// wait until reseed is done
		while(!CheckRndState(hRnd))
			Sleep(100);
	}
	
#if 0
	{
		unsigned int entropy;
		RndCalcEntropy(hRnd, &entropy);
		g10_log_info("Entropy count of RND1: is %d\n", entropy );
	}
#endif

  while( length ) 
  {
	  int nbytes = length < sizeof(buffer)? length : sizeof(buffer);
	  if(!ReadFile(hRnd, buffer, nbytes, &n, NULL))
		  g10_log_fatal("read error on random device: %d\n", GetLastError());
	  (*add)( buffer, n, requester );
	  length -= n;
  }
  memset(buffer, 0, sizeof(buffer) );
  
  return 0; /* success */
}

#endif /*USE_RNDPOETRY*/
