
#include "cehelp.h"



#define getpid() (pid_t)GetCurrentProcessId()

int umask(x)
{
	return x;
}

int ftruncate( int handle, long size )
{
	SetFilePointer((HANDLE)handle, 0, NULL, FILE_BEGIN);
	return SetEndOfFile((HANDLE)handle) == FALSE;
}

int open( char *path, int oflag,... )
{
	int filepos;                    /* length of file - 1 */
	char ch;                     /* character at end of file */
	HANDLE osfh;                    /* OS handle of opened file */
	DWORD fileaccess;               /* OS file access (requested) */
	DWORD fileshare;                /* OS file sharing mode */
	DWORD filecreate;               /* OS method of opening/creating */
	DWORD fileattrib;               /* OS file attribute flags */
	wchar_t wpath[MAX_PATH];	
	


	fileaccess = GENERIC_READ | GENERIC_WRITE;
	fileshare = FILE_SHARE_READ | FILE_SHARE_WRITE;

	/* figure out binary/text mode */
	if ((oflag & _O_BINARY) == 0)
			/*
			* decode the access flags
		*/
        switch( oflag & (O_RDONLY | O_WRONLY | O_RDWR) ) {
			
		case O_RDONLY:         /* read access */
			fileaccess = GENERIC_READ;
			fileshare = FILE_SHARE_READ;
			break;
		case O_WRONLY:         /* write access */
			fileaccess = GENERIC_WRITE;
			fileshare = FILE_SHARE_WRITE;
			break;
		case O_RDWR:           /* read and write access */
			fileaccess = GENERIC_READ | GENERIC_WRITE;
			fileshare = FILE_SHARE_READ | FILE_SHARE_WRITE;
			break;
		default:                /* error, bad oflag */
			errno = EINVAL;
			return -1;
        }
		
  	
        /*
		* decode open/create method flags
		*/
        switch ( oflag & (O_CREAT | O_EXCL | O_TRUNC) ) {
		case 0:
		case O_EXCL:                   // ignore EXCL w/o CREAT
			filecreate = OPEN_EXISTING;
			break;
			
		case O_CREAT:
			filecreate = OPEN_ALWAYS;
			break;
			
		case O_CREAT | O_EXCL:
		case O_CREAT | O_TRUNC | O_EXCL:
			filecreate = CREATE_NEW;
			break;
			
		case O_TRUNC:
		case O_TRUNC | O_EXCL:        // ignore EXCL w/o CREAT
			filecreate = TRUNCATE_EXISTING;
			break;
			
		case O_CREAT | O_TRUNC:
			filecreate = CREATE_ALWAYS;
			break;
			
		default:
			// this can't happen ... all cases are covered
			errno = EINVAL;
			return -1;
        }
		
		
        /*
		* decode file attribute flags if _O_CREAT was specified
		*/
        fileattrib = FILE_ATTRIBUTE_NORMAL;     /* default */
		
        /*
		* try to open/create the file
		*/
		MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, MAX_PATH);
        if ( (osfh = CreateFileW( wpath,
			fileaccess,
			fileshare,
			NULL,
			filecreate,
			fileattrib,
			NULL ))
			== (HANDLE)0xffffffff )
        {
		/*
		* OS call to open/create file failed!
		*/
            return -1;                      /* return error to caller */
        }
		
       return (int)osfh;                      /* return handle */
}

DWORD lseek( int handle, long offset, int origin )
{
	return SetFilePointer((HANDLE)handle, offset, NULL, origin);
}

int write( int handle, const void *buffer, unsigned int count )
{
	DWORD dwWritten;
	WriteFile((HANDLE)handle, buffer, count, &dwWritten, NULL);
	return dwWritten;
}

int read( int handle, void *buffer, unsigned int count )
{
	DWORD dwRead;

	if(handle == (int)fileno(stdin))
	{
		ReadConsole(GetStdHandle(STD_INPUT_HANDLE), buffer, count, &dwRead, NULL);
		
	}else
	{
		ReadFile((HANDLE)handle, buffer, count, &dwRead,NULL);
	}
	return dwRead;
}


FILE *fdopen( int handle, const char *mode )
{
	char FileName[MAX_PATH];
	BY_HANDLE_FILE_INFORMATION FileInformation;
	CEOIDINFO oidInfo;
	if(!GetFileInformationByHandle((HANDLE)handle, &FileInformation))
		return NULL;
	if(!CeOidGetInfo(FileInformation.dwOID, &oidInfo))
		return NULL;
	WideCharToMultiByte(CP_ACP, 0, oidInfo.infFile.szFileName, -1, FileName, MAX_PATH, NULL, NULL);
	CloseHandle((HANDLE)handle);
	return fopen(FileName, mode);
}

int rmdir( const char *dirname )
{
	wchar_t wdirname[MAX_PATH];
	MultiByteToWideChar(CP_ACP,0, dirname,-1, wdirname, MAX_PATH);
	return RemoveDirectoryW(wdirname);
}

int mkdir( const char *dirname )
{
	wchar_t wdirname[MAX_PATH];
	MultiByteToWideChar(CP_ACP,0, dirname,-1, wdirname, MAX_PATH);
	return (CreateDirectoryW(wdirname, NULL) == FALSE);
}


close(int handle)
{
	return !CloseHandle((HANDLE)handle);
}


// _setmode from Windows CE is buggy!
// it crashes with datatype alignment error on SH3 and MIPS
// and fails on ARM 

int setmode ( void* handle, int mode )
{
	wchar_t PathName[MAX_PATH];
	DWORD PathLen = MAX_PATH;
	if(handle == fileno(stdin))
	{
		GetStdioPathW(0, PathName, &PathLen);
		if(mode == _O_BINARY)
			_wfreopen(PathName, L"wb", stdin);
		else
			_wfreopen(PathName, L"w", stdin);
	}else 
		if(handle == fileno(stdout))
	{
		GetStdioPathW(1, PathName, &PathLen);
		if(mode == _O_BINARY)
			_wfreopen(PathName, L"wb", stdout);
		else
			_wfreopen(PathName, L"w", stdout);
	}
	return 0;
}


